package com.tzld.longarticle.recommend.server.service.rank.strategy;


import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.service.AccountContentPoolConfigService;
import com.tzld.longarticle.recommend.server.service.rank.RankItem;
import com.tzld.longarticle.recommend.server.service.rank.RankParam;
import com.tzld.longarticle.recommend.server.service.rank.RankResult;
import com.tzld.longarticle.recommend.server.service.rank.RankStrategy;
import com.tzld.longarticle.recommend.server.service.score.ScoreResult;
import com.tzld.longarticle.recommend.server.service.score.ScoreService;
import com.tzld.longarticle.recommend.server.service.score.strategy.SimilarityStrategy;
import com.tzld.longarticle.recommend.server.service.score.strategy.ViewCountStrategy;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import com.tzld.longarticle.recommend.server.util.JSONUtils;
import com.tzld.longarticle.recommend.server.util.feishu.FeishuMessageSender;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.RandomUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author dyp
 */
@Service
@Slf4j
public class DefaultRankStrategy implements RankStrategy {

    @Autowired
    private ScoreService scoreService;
    @Autowired
    private AccountContentPoolConfigService accountContentPoolConfigService;

    public RankResult rank(RankParam param) {
        List<Content> result = new ArrayList<>();
        //log.info("RankParam {}", JSONUtils.toJson(param));
        ScoreResult scoreResult = scoreService.score(RankStrategy.convertToScoreParam(param));
        //log.info("ScoreResult {}", JSONUtils.toJson(scoreResult));

        Map<String, Map<String, Double>> scoreMap = scoreResult.getScoreMap();

        List<RankItem> items = CommonCollectionUtils.toList(param.getContents(), c -> {
            RankItem item = new RankItem();
            item.setContent(c);
            c.setScoreMap(scoreMap.get(c.getId()));
            item.setScoreMap(scoreMap.get(c.getId()));
            return item;

        });

        // 1 排序
        String[] contentPools = accountContentPoolConfigService.getContentPools(param.getAccountName());
        Map<String, List<RankItem>> itemMap = new HashMap<>();
        for (RankItem c : items) {
            List<RankItem> data = itemMap.computeIfAbsent(c.getContent().getContentPoolType(), k -> new ArrayList<>());
            data.add(c);
        }
        List<RankItem> sortedItems = new ArrayList<>();
        for (int i = 0; i < contentPools.length; i++) {
            if (i == 1) {
                // 播放量排序
                List<RankItem> data = itemMap.get(contentPools[i]);
                if (CollectionUtils.isNotEmpty(data)) {
                    Collections.sort(data, (o1, o2) -> -Double.compare(
                            o1.getScore(ViewCountStrategy.class.getSimpleName()),
                            o2.getScore(ViewCountStrategy.class.getSimpleName())));
                    sortedItems.addAll(data);
                }
            } else {
                // 相似排序
                List<RankItem> data = itemMap.get(contentPools[i]);
                if (CollectionUtils.isNotEmpty(data)) {
                    Collections.sort(data, (o1, o2) -> -Double.compare(
                            o1.getScore(SimilarityStrategy.class.getSimpleName()),
                            o2.getScore(SimilarityStrategy.class.getSimpleName())));
                    sortedItems.addAll(data);
                }
            }
        }
        // 相似度评分为0 报警返回
        if (CollectionUtils.isNotEmpty(items) && items.get(0).getScoreMap().get(SimilarityStrategy.class.getSimpleName()) == 0) {
            FeishuMessageSender.sendWebHookMessage("07026a9f-43f5-448b-ba40-a8d71bd6e634",
                    "内容评分为0\n"
                            + "ghId: " + param.getGhId() + "\n"
                            + "账号名称: " + param.getAccountName() + "\n"
                            + "策略: " + this.getClass().getSimpleName());
            return new RankResult(result);
        }

        log.info("Sort result {}", JSONUtils.toJson(sortedItems));
        List<Content> contents = CommonCollectionUtils.toList(sortedItems, RankItem::getContent);

        // 3 相似去重
//        contents = deduplication(contents);
        log.info("Deduplication {}", JSONUtils.toJson(contents));
        String[] publishPool = Arrays.copyOf(contentPools, contentPools.length);

        // 4 文章按照内容池分组
        Map<String, List<Content>> contentMap = new HashMap<>();
        for (Content c : contents) {
            List<Content> data = contentMap.computeIfAbsent(c.getContentPoolType(), k -> new ArrayList<>());
            data.add(c);
        }
        log.info("ContentMap {}", JSONUtils.toJson(contentMap));
        // 5 按位置选文章
        // 头
        List<Content> pool = contentMap.get(contentPools[0]);
        if (CollectionUtils.isNotEmpty(pool)) {
            result.add(pool.get(RandomUtils.nextInt(0, Math.min(pool.size(), 5))));
        } else {
            FeishuMessageSender.sendWebHookMessage("07026a9f-43f5-448b-ba40-a8d71bd6e634",
                    "内容池1为空\n"
                            + "ghId: " + param.getGhId() + "\n"
                            + "账号名称: " + param.getAccountName() + "\n"
                            + "内容池: " + contentPools[0] + "\n"
                            + "策略: " + this.getClass().getSimpleName());
            return new RankResult(result);
        }

        // 次
        pool = contentMap.get(contentPools[1]);
        if (CollectionUtils.isNotEmpty(pool)) {
            result.add(pool.get(0));
            if (result.size() == 1 && pool.size() > 1) {
                publishPool[0] = contentPools[1];
                result.add(pool.get(1));
            }
        }

        // 3-8
        pool = contentMap.get(contentPools[2]);
        if (CollectionUtils.isNotEmpty(pool)) {
            result.addAll(pool.subList(0, Math.min(pool.size(), param.getSize() - result.size())));
        }

        RankStrategy.deduplication(result, contentMap, publishPool);

        return new RankResult(result);
    }

}
