package com.tzld.longarticle.recommend.server.service.score.strategy;

import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.model.dto.ContentHisPublishArticle;
import com.tzld.longarticle.recommend.server.repository.crawler.AccountAvgInfoRepository;
import com.tzld.longarticle.recommend.server.model.entity.crawler.AccountAvgInfo;
import com.tzld.longarticle.recommend.server.service.AccountContentPoolConfigService;
import com.tzld.longarticle.recommend.server.service.AccountIndexAvgViewCountService;
import com.tzld.longarticle.recommend.server.service.score.Score;
import com.tzld.longarticle.recommend.server.service.score.ScoreParam;
import com.tzld.longarticle.recommend.server.service.score.ScoreStrategy;
import com.tzld.longarticle.recommend.server.util.MathUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Component
@Slf4j
public class ViewCountRateStrategy implements ScoreStrategy {

    @Autowired
    AccountIndexAvgViewCountService accountIndexAvgViewCountService;
    @Autowired
    AccountContentPoolConfigService accountContentPoolConfigService;
    @Autowired
    AccountAvgInfoRepository accountAvgInfoRepository;

    @Override
    public List<Score> score(ScoreParam param) {
        long start = System.currentTimeMillis();
        List<Score> scores = new ArrayList<>();
        String[] contentPools = accountContentPoolConfigService.getContentPools(param.getAccountName());
        List<AccountAvgInfo> avgInfoList = accountAvgInfoRepository.getAllByGhIdEqualsAndStatusEquals(param.getGhId(), 1);
        double avgViewCountFirst = accountIndexAvgViewCountService.getAvgReadCountByDB(avgInfoList, param.getGhId(), 1);
        double avgViewCountSecond = accountIndexAvgViewCountService.getAvgReadCountByDB(avgInfoList, param.getGhId(), 2);
        double avgViewCountThird = accountIndexAvgViewCountService.getAvgReadCountByDB(avgInfoList, param.getGhId(), 3);
        // 缺省头条均值设置为2w，次条为1w
        if (avgViewCountFirst < 10) {
            avgViewCountFirst = 20000D;
            avgViewCountSecond = 10000D;
            avgViewCountThird = 400D;
        }
        for (Content content : param.getContents()) {
            for (int i = 0; i < contentPools.length; i++) {
                if (!contentPools[i].equals(content.getContentPoolType())) {
                    continue;
                }
                double avgViewCountPos = accountIndexAvgViewCountService.getAvgReadCountByDB(avgInfoList, param.getGhId(), i + 1);
                // 缺省头条均值设置为2w，次条为1w
                if (avgViewCountPos < 10) {
                    if (i == 0) {
                        avgViewCountPos = 20000D;
                    } else if (i == 1) {
                        avgViewCountPos = 10000D;
                    } else {
                        avgViewCountPos = 400D;
                    }
                }
                double showViewCountSum = 0D;
                double avgViewCountSum = 0D;
                double showViewCountSumFirst = 0D;
                double avgViewCountSumFirst = 0D;
                double showViewCountSumSecond = 0D;
                double avgViewCountSumSecond = 0D;
                double maxAvgViewCount = 0D;
                for (ContentHisPublishArticle hisItem : content.getHisPublishArticleList()) {
                    if (hisItem.isInnerAccount() && Objects.nonNull(hisItem.getViewCount())
                            && hisItem.getViewCount() > 0 && Objects.nonNull(hisItem.getAvgViewCount())
                            && hisItem.getAvgViewCount() > 0) {
                        maxAvgViewCount = Math.max(maxAvgViewCount, hisItem.getAvgViewCount());
                        if (hisItem.getItemIndex() == 1) {
                            showViewCountSumFirst += hisItem.getViewCount();
                            avgViewCountSumFirst += hisItem.getAvgViewCount();
                        } else if (hisItem.getItemIndex() == 2) {
                            if (Objects.nonNull(hisItem.getFirstViewCount()) &&  hisItem.getFirstViewCount() > 0 &&
                                    Objects.nonNull(hisItem.getFirstViewCountRate()) && hisItem.getFirstViewCountRate() > 0) {
                                showViewCountSumSecond += hisItem.getViewCount();
                                if (hisItem.getFirstViewCountRate() > 1) {
                                    // 对于头条均值倍数大于1的情况，次条均值线性增加，用于debias；
                                    // TODO: 对于小于1的情况，是否要减去？
                                    avgViewCountSumSecond += hisItem.getAvgViewCount() * hisItem.getFirstViewCountRate();
                                } else {
                                    avgViewCountSumSecond += hisItem.getAvgViewCount();
                                }
                            }
                        } else {
                            if (Objects.nonNull(hisItem.getFirstViewCount()) && hisItem.getFirstViewCount() > 0
                                    && Objects.nonNull(hisItem.getFirstViewCountRate()) && hisItem.getFirstViewCountRate() > 0) {
                                showViewCountSum += hisItem.getViewCount();
                                if (hisItem.getFirstViewCountRate() > 1) {
                                    // 对于头条均值倍数大于1的情况，次条均值线性增加，用于debias；
                                    // TODO: 对于小于1的情况，是否要减去？
                                    avgViewCountSum += hisItem.getAvgViewCount() * hisItem.getFirstViewCountRate();
                                } else {
                                    avgViewCountSum += hisItem.getAvgViewCount();
                                }
                            }
                        }
                    }
                }
                double viewCountRate = 0D; // 设置默认值
                double bigRateW = 1D;
                // 如果有头条反馈数据，优先选取头条反馈数据；
                if (showViewCountSumFirst > 0) {
                    showViewCountSum = showViewCountSumFirst;
                    avgViewCountSum = avgViewCountSumFirst;
                } else if (showViewCountSumSecond > 0) {
                    showViewCountSum = showViewCountSumSecond;
                    avgViewCountSum = avgViewCountSumSecond;
                    // 如果是大号头条，则降权
                    if (avgViewCountFirst >= 3000 && i == 0) {
                        bigRateW = 0.001D;
                    }
                }
                // 均值倍数
                if (avgViewCountSum > 0) {
                    viewCountRate = showViewCountSum / avgViewCountSum;
                }
                // 置信度
                double viewCountRateW = MathUtils.sigmoid(avgViewCountSum, 0.0005, avgViewCountPos);
                double viewCountRateScore = 0;

                if (viewCountRate > 0) {
                    // 最终分数 = 置信度 * 均值倍数
                    if (viewCountRate > 1 && bigRateW < 1) {
                        // 如果是大号头条，则降权
                        viewCountRateScore = viewCountRateW * ((viewCountRate - 1) * bigRateW + 1);
                    } else {
                        viewCountRateScore = viewCountRateW * viewCountRate;
                    }
                }
                Score score = new Score();
                score.setStrategy(this);
                score.setContentId(content.getId());
                score.setScore(viewCountRateScore);
                scores.add(score);
                break;
            }
        }
        return scores;
    }
}
